@@ -1,8 +1,27 @@
 # -*- coding: UTF-8 -*-
+##############################################################################
+#
+#    OERPLib
+#    Copyright (C) 2011-2013 Sébastien Alix.
+#
+#    This program is free software: you can redistribute it and/or modify
+#    it under the terms of the GNU Lesser General Public License as published
+#    by the Free Software Foundation, either version 3 of the License, or
+#    (at your option) any later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU Lesser General Public License for more details.
+#
+#    You should have received a copy of the GNU Lesser General Public License
+#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+##############################################################################
 
 import xmlrpclib
 
-from oerplib.rpc import socket_netrpc, xmlrpclib_custom, error
+from oerplib.rpc import netrpclib, xmlrpclib_custom, error
 
 
 class ServiceXMLRPC(object):
@@ -15,8 +34,8 @@ class ServiceXMLRPC(object):
         def rpc_method(*args):
             try:
                 self._sock = xmlrpclib_custom.TimeoutServerProxy(
-                        self._url, allow_none=True,
-                        timeout=self._connector.timeout)
+                    self._url, allow_none=True,
+                    timeout=self._connector.timeout)
                 sock_method = getattr(self._sock, method, False)
                 return sock_method(*args)
             #NOTE: exception raised with these kind of requests:
@@ -44,16 +63,16 @@ class ServiceNetRPC(object):
     def __getattr__(self, method):
         def rpc_method(*args):
             try:
-                sock = socket_netrpc.NetRPC(timeout=self._connector.timeout)
+                sock = netrpclib.NetRPC(timeout=self._connector.timeout)
                 sock.connect(self._server, self._port)
-                sock.send((self._name, method, )+args)
+                sock.send((self._name, method, ) + args)
                 result = sock.receive()
                 sock.disconnect()
                 return result
             #NOTE: exception raised with these kind of requests:
             #   - execute('fake.model', 'search', [])
             #   - execute('sale.order', 'fake_method')
-            except socket_netrpc.NetRPCError as exc:
+            except netrpclib.NetRPCError as exc:
                 # faultCode: error message
                 # faultString: OpenERP server traceback (following the OpenERP
                 # server version used, a bad request can produce a
