@@ -1,4 +1,23 @@
 # -*- coding: UTF-8 -*-
+##############################################################################
+#
+#    OERPLib
+#    Copyright (C) 2011-2013 Sébastien Alix.
+#
+#    This program is free software: you can redistribute it and/or modify
+#    it under the terms of the GNU Lesser General Public License as published
+#    by the Free Software Foundation, either version 3 of the License, or
+#    (at your option) any later version.
+#
+#    This program is distributed in the hope that it will be useful,
+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#    GNU Lesser General Public License for more details.
+#
+#    You should have received a copy of the GNU Lesser General Public License
+#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+##############################################################################
 """This module contains classes representing the fields supported by OpenObject.
 A field is a Python descriptor which defines getter/setter methods for
 its related attribute.
@@ -8,7 +27,9 @@ import datetime
 from oerplib import error
 from oerplib.service.osv import browse
 
+
 def is_int(value):
+    """Return `True` if ``value`` is an integer."""
     if isinstance(value, bool):
         return False
     try:
@@ -18,6 +39,43 @@ def is_int(value):
         return False
 
 
+def oerp_tuple_in(iterable):
+    """Return `True` if `iterable` contains an expected tuple like
+    ``(6, 0, IDS)`` (and so on).
+
+        >>> oerp_tuple_in([0, 1, 2])        # Simple list
+        False
+        >>> oerp_tuple_in([(6, 0, [42])])   # List of tuples
+        True
+        >>> oerp_tuple_in([[1, 42]])        # List of lists
+        True
+    """
+    if not iterable:
+        return False
+    def is_oerp_tuple(elt):
+        try:
+            return elt[:1][0] in [1, 2, 3, 4, 5] \
+                    or elt[:2] in [(6, 0), [6, 0], (0, 0), [0, 0]]
+        except:
+            return False
+    return any(is_oerp_tuple(elt) for elt in iterable)
+
+
+def records2ids(iterable):
+    """Replace `browse_records` contained in `iterable` by their
+    corresponding IDs:
+
+        >>> groups = list(oerp.user.groups_id)
+        >>> records2ids(groups)
+        [1, 2, 3, 14, 17, 18, 19, 7, 8, 9, 5, 20, 21, 22, 23]
+    """
+    def record2id(elt):
+        if isinstance(elt, browse.BrowseRecord):
+            return elt.id
+        return elt
+    return [record2id(elt) for elt in iterable]
+
+
 class BaseField(object):
     """Field which all other fields inherit.
     Manage common metadata.
@@ -58,19 +116,19 @@ class BaseField(object):
         """Check the validity of a value for the field."""
         #if self.readonly:
         #    raise error.Error(
-        #        u"'{field_name}' field is readonly".format(
+        #        "'{field_name}' field is readonly".format(
         #            field_name=self.name))
         if value and self.size:
             if not isinstance(value, basestring):
-                raise ValueError(u"Value supplied has to be a string")
+                raise ValueError("Value supplied has to be a string")
             if len(value) > self.size:
                 raise ValueError(
-                    u"Lenght of the '{field_name}' is limited to {size}".format(
+                    "Lenght of the '{field_name}' is limited to {size}".format(
                         field_name=self.name,
                         size=self.size))
         if not value and self.required:
             raise ValueError(
-                u"'{field_name}' field require a value".format(
+                "'{field_name}' field is required".format(
                     field_name=self.name))
         return value
 
@@ -82,24 +140,25 @@ class SelectionField(BaseField):
         self.selection = 'selection' in data and data['selection'] or False
 
     def __get__(self, instance, owner):
-        return getattr(instance, "_{0}".format(self.name))
+        if self.name in instance.__data__['updated_values']:
+            return instance.__data__['updated_values'][self.name]
+        return instance.__data__['values'][self.name]
 
     def __set__(self, instance, value):
         value = self.check_value(value)
-        setattr(instance, "_{0}".format(self.name), value)
-        instance.__data__['fields_updated'].append(self.name)
+        instance.__data__['updated_values'][self.name] = value
 
     def check_value(self, value):
         super(SelectionField, self).check_value(value)
         selection = [val[0] for val in self.selection]
         if value and value not in selection:
             raise ValueError(
-                u"The value '{value}' supplied doesn't match with the possible "
-                u"values '{selection}' for the '{field_name}' field".format(
+                "The value '{value}' supplied doesn't match with the possible "
+                "values '{selection}' for the '{field_name}' field".format(
                     value=value,
                     selection=selection,
                     field_name=self.name,
-            ))
+                ))
         return value
 
 
@@ -108,25 +167,59 @@ class Many2ManyField(BaseField):
     def __init__(self, osv, name, data):
         super(Many2ManyField, self).__init__(osv, name, data)
         self.relation = 'relation' in data and data['relation'] or False
-        self.context = 'context' in data and data['context'] or False
+        self.context = 'context' in data and data['context'] or {}
         self.domain = 'domain' in data and data['domain'] or False
 
     def __get__(self, instance, owner):
-        if getattr(instance, "_{0}".format(self.name)):
-            return [instance.__oerp__.browse(self.relation, o_id)
-                    for o_id in getattr(instance, "_{0}".format(self.name))]
-        return []
+        """Return a generator to iterate on ``browse_record`` instances."""
+        ids = None
+        if instance.__data__['values'][self.name]:
+            ids = instance.__data__['values'][self.name][:]
+        # None value => get the value on the fly
+        if ids is None:
+            orig_ids = instance.__oerp__.read(
+                instance.__osv__['name'],
+                [instance.id], [self.name])[0][self.name]
+            instance.__data__['values'][self.name] = orig_ids
+            ids = orig_ids and orig_ids[:] or []
+        # Take updated values into account
+        if self.name in instance.__data__['updated_values']:
+            ids = ids or []
+            values = instance.__data__['updated_values'][self.name]
+            # Handle OERP tuples to update 'ids'
+            for value in values:
+                if value[0] == 6 and value[2]:
+                    ids = value[2]
+                elif value[0] == 5:
+                    ids = []
+                elif value[0] == 4 and value[1] and value[1] not in ids:
+                    ids.append(value[1])
+                elif value[0] == 3 and value[1] and value[1] in ids:
+                    ids.remove(value[1])
+        if ids:
+            context = instance.__data__['context'].copy()
+            context.update(self.context)
+            return browse.BrowseRecordIterator(
+                model=instance.__oerp__.get(self.relation),
+                ids=ids,
+                context=context,
+                parent=instance,
+                parent_field=self)
+        return iter(())
 
     def __set__(self, instance, value):
         value = self.check_value(value)
-        setattr(instance, "_{0}".format(self.name), value)
-        instance.__data__['fields_updated'].append(self.name)
+        if value and not oerp_tuple_in(value):
+            value = [(6, 0, records2ids(value))]
+        elif not value:
+            value = [(5, )]
+        instance.__data__['updated_values'][self.name] = value
 
     def check_value(self, value):
         if value:
             if not isinstance(value, list):
                 raise ValueError(
-                        u"The value supplied has to be a list or 'False'")
+                    "The value supplied has to be a list or 'False'")
         return super(Many2ManyField, self).check_value(value)
 
 
@@ -135,15 +228,25 @@ class Many2OneField(BaseField):
     def __init__(self, osv, name, data):
         super(Many2OneField, self).__init__(osv, name, data)
         self.relation = 'relation' in data and data['relation'] or False
-        self.context = 'context' in data and data['context'] or False
+        self.context = 'context' in data and data['context'] or {}
         self.domain = 'domain' in data and data['domain'] or False
 
     def __get__(self, instance, owner):
-        if getattr(instance, "_{0}".format(self.name)):
+        id_ = instance.__data__['values'][self.name]
+        if self.name in instance.__data__['updated_values']:
+            id_ = instance.__data__['updated_values'][self.name]
+            # FIXME if id_ is a browse_record
+        # None value => get the value on the fly
+        if id_ is None:
+            id_ = instance.__oerp__.read(
+                instance.__osv__['name'],
+                [instance.id], [self.name])[0][self.name]
+            instance.__data__['values'][self.name] = id_
+        if id_:
+            context = instance.__data__['context'].copy()
+            context.update(self.context)
             return instance.__class__.__oerp__.browse(
-                    self.relation,
-                    getattr(instance, "_{0}".format(self.name))[0]
-                )
+                self.relation, id_[0], context)
         return False
 
     def __set__(self, instance, value):
@@ -154,21 +257,21 @@ class Many2OneField(BaseField):
         elif value in [None, False]:
             o_rel = False
         else:
-            raise ValueError(u"Value supplied has to be an integer, "
-                             u"a browse_record object or False.")
+            raise ValueError("Value supplied has to be an integer, "
+                             "a browse_record object or False.")
         o_rel = self.check_value(o_rel)
-        setattr(instance, "_{0}".format(self.name), o_rel and [o_rel.id, False])
-        instance.__data__['fields_updated'].append(self.name)
+        instance.__data__['updated_values'][self.name] = \
+            o_rel and [o_rel.id, False]
 
     def check_value(self, value):
         super(Many2OneField, self).check_value(value)
         if value and value.__osv__['name'] != self.relation:
             raise ValueError(
-                (u"Instance of '{osv_name}' supplied doesn't match with the " +\
-                 u"relation '{relation}' of the '{field_name}' field.").format(
-                    osv_name=value.__osv__['name'],
-                    relation=self.relation,
-                    field_name=self.name))
+                ("Instance of '{model}' supplied doesn't match with the " +
+                 "relation '{relation}' of the '{field_name}' field.").format(
+                     model=value.__osv__['name'],
+                     relation=self.relation,
+                     field_name=self.name))
         return value
 
 
@@ -177,64 +280,107 @@ class One2ManyField(BaseField):
     def __init__(self, osv, name, data):
         super(One2ManyField, self).__init__(osv, name, data)
         self.relation = 'relation' in data and data['relation'] or False
-        self.context = 'context' in data and data['context'] or False
+        self.context = 'context' in data and data['context'] or {}
         self.domain = 'domain' in data and data['domain'] or False
 
     def __get__(self, instance, owner):
-        if getattr(instance, "_{0}".format(self.name)):
-            return [instance.__oerp__.browse(self.relation, o_id)
-                    for o_id in getattr(instance, "_{0}".format(self.name))]
-        return []
+        """Return a generator to iterate on ``browse_record`` instances."""
+        ids = None
+        if instance.__data__['values'][self.name]:
+            ids = instance.__data__['values'][self.name][:]
+        # None value => get the value on the fly
+        if ids is None:
+            orig_ids = instance.__oerp__.read(
+                instance.__osv__['name'],
+                [instance.id], [self.name])[0][self.name]
+            instance.__data__['values'][self.name] = orig_ids
+            ids = orig_ids and orig_ids[:] or []
+        # Take updated values into account
+        if self.name in instance.__data__['updated_values']:
+            ids = ids or []
+            values = instance.__data__['updated_values'][self.name]
+            # Handle OERP tuples to update 'ids'
+            for value in values:
+                if value[0] == 6 and value[2]:
+                    ids = value[2]
+                elif value[0] == 5:
+                    ids = []
+                elif value[0] == 4 and value[1] and value[1] not in ids:
+                    ids.append(value[1])
+                elif value[0] == 3 and value[1] and value[1] in ids:
+                    ids.remove(value[1])
+        if ids:
+            context = instance.__data__['context'].copy()
+            context.update(self.context)
+            return browse.BrowseRecordIterator(
+                model=instance.__oerp__.get(self.relation),
+                ids=ids,
+                context=context,
+                parent=instance,
+                parent_field=self)
+        return iter(())
 
     def __set__(self, instance, value):
         value = self.check_value(value)
-        setattr(instance, "_{0}".format(self.name), value)
-        instance.__data__['fields_updated'].append(self.name)
+        if value and not oerp_tuple_in(value):
+            value = [(6, 0, records2ids(value))]
+        elif not value:
+            value = [(5, )]
+        instance.__data__['updated_values'][self.name] = value
 
     def check_value(self, value):
         if value:
             if not isinstance(value, list):
                 raise ValueError(
-                        u"The value supplied has to be a list or 'False'")
+                    "The value supplied has to be a list or 'False'")
         return super(One2ManyField, self).check_value(value)
 
 
 class ReferenceField(BaseField):
-    """.. versionadded:: 0.6.0
+    """.. versionadded:: 0.6
     Represent the OpenObject 'fields.reference'.
     """
     def __init__(self, osv, name, data):
         super(ReferenceField, self).__init__(osv, name, data)
-        self.context = 'context' in data and data['context'] or False
+        self.context = 'context' in data and data['context'] or {}
         self.domain = 'domain' in data and data['domain'] or False
         self.selection = 'selection' in data and data['selection'] or False
 
     def __get__(self, instance, owner):
-        if getattr(instance, "_{0}".format(self.name)):
-            value = getattr(instance, "_{0}".format(self.name))
+        value = instance.__data__['values'][self.name]
+        if self.name in instance.__data__['updated_values']:
+            value = instance.__data__['updated_values'][self.name]
+        # None value => get the value on the fly
+        if value is None:
+            value = instance.__oerp__.read(
+                instance.__osv__['name'],
+                [instance.id], [self.name])[0][self.name]
+            instance.__data__['values'][self.name] = value
+        if value:
             relation, sep, o_id = value.rpartition(',')
             relation = relation.strip()
             o_id = int(o_id.strip())
             if relation and o_id:
+                context = instance.__data__['context'].copy()
+                context.update(self.context)
                 return instance.__class__.__oerp__.browse(
-                        relation, o_id)
+                    relation, o_id, context)
         return False
 
     def __set__(self, instance, value):
         value = self.check_value(value)
-        setattr(instance, "_{0}".format(self.name), value)
-        instance.__data__['fields_updated'].append(self.name)
+        instance.__data__['updated_values'][self.name] = value
 
     def _check_relation(self, relation):
         selection = [val[0] for val in self.selection]
         if relation not in selection:
             raise ValueError(
-                (u"The value '{value}' supplied doesn't match with the possible"
-                 u" values '{selection}' for the '{field_name}' field").format(
-                    value=relation,
-                    selection=selection,
-                    field_name=self.name,
-            ))
+                ("The value '{value}' supplied doesn't match with the possible"
+                 " values '{selection}' for the '{field_name}' field").format(
+                     value=relation,
+                     selection=selection,
+                     field_name=self.name,
+                 ))
         return relation
 
     def check_value(self, value):
@@ -250,33 +396,35 @@ class ReferenceField(BaseField):
             o_id = o_id.strip()
             #o_rel = instance.__class__.__oerp__.browse(relation, o_id)
             if not relation or not is_int(o_id):
-                raise ValueError(u"String not well formatted, expecting "
-                                 u"'{relation},{id}' format")
+                raise ValueError("String not well formatted, expecting "
+                                 "'{relation},{id}' format")
             self._check_relation(relation)
         else:
-            raise ValueError(u"Value supplied has to be a string or"
-                             u" a browse_record object.")
+            raise ValueError("Value supplied has to be a string or"
+                             " a browse_record object.")
         return value
 
 
 class DateField(BaseField):
     """Represent the OpenObject 'fields.data'"""
     pattern = "%Y-%m-%d"
+
     def __init__(self, osv, name, data):
         super(DateField, self).__init__(osv, name, data)
 
     def __get__(self, instance, owner):
-        value = getattr(instance, "_{0}".format(self.name))
+        value = instance.__data__['values'][self.name]
+        if self.name in instance.__data__['updated_values']:
+            value = instance.__data__['updated_values'][self.name]
         try:
             res = datetime.datetime.strptime(value, self.pattern).date()
-        except Exception: # ValueError, TypeError
+        except Exception:  # ValueError, TypeError
             res = value
         return res
 
     def __set__(self, instance, value):
         value = self.check_value(value)
-        setattr(instance, "_{0}".format(self.name), value)
-        instance.__data__['fields_updated'].append(self.name)
+        instance.__data__['updated_values'][self.name] = value
 
     def check_value(self, value):
         super(DateField, self).check_value(value)
@@ -293,28 +441,30 @@ class DateField(BaseField):
             return value
         else:
             raise ValueError(
-                    "Expecting a datetime.date object or basestring")
+                "Expecting a datetime.date object or basestring")
         return value
 
 
 class DateTimeField(BaseField):
     """Represent the OpenObject 'fields.datetime'"""
     pattern = "%Y-%m-%d %H:%M:%S"
+
     def __init__(self, osv, name, data):
         super(DateTimeField, self).__init__(osv, name, data)
 
     def __get__(self, instance, owner):
-        value = getattr(instance, "_{0}".format(self.name))
+        value = instance.__data__['values'][self.name]
+        if self.name in instance.__data__['updated_values']:
+            value = instance.__data__['updated_values'][self.name]
         try:
             res = datetime.datetime.strptime(value, self.pattern)
-        except Exception: # ValueError, TypeError
+        except Exception:  # ValueError, TypeError
             res = value
         return res
 
     def __set__(self, instance, value):
         value = self.check_value(value)
-        setattr(instance, "_{0}".format(self.name), value)
-        instance.__data__['fields_updated'].append(self.name)
+        instance.__data__['updated_values'][self.name] = value
 
     def check_value(self, value):
         super(DateTimeField, self).check_value(value)
@@ -331,7 +481,7 @@ class DateTimeField(BaseField):
             return value
         else:
             raise ValueError(
-                    "Expecting a datetime.datetime object or basestring")
+                "Expecting a datetime.datetime object or basestring")
         return value
 
 
@@ -348,12 +498,13 @@ class ValueField(BaseField):
         super(ValueField, self).__init__(osv, name, data)
 
     def __get__(self, instance, owner):
-        return getattr(instance, "_{0}".format(self.name))
+        if self.name in instance.__data__['updated_values']:
+            return instance.__data__['updated_values'][self.name]
+        return instance.__data__['values'][self.name]
 
     def __set__(self, instance, value):
         value = self.check_value(value)
-        setattr(instance, "_{0}".format(self.name), value)
-        instance.__data__['fields_updated'].append(self.name)
+        instance.__data__['updated_values'][self.name] = value
 
 
 def generate_field(osv, name, data):
@@ -377,12 +528,12 @@ def generate_field(osv, name, data):
         field = DateField(osv, name, data)
     elif data['type'] == 'datetime':
         field = DateTimeField(osv, name, data)
-    elif data['type'] in ['char', 'float', 'integer',
-                          'boolean', 'text', 'binary']:
+    elif data['type'] in ['char', 'float', 'integer', 'integer_big',
+                          'boolean', 'text', 'binary', 'html']:
         field = ValueField(osv, name, data)
     else:
-        txt = (u"Can't instanciate the field '{field_name}', "
-               u"'{field_type}' type unknown")
+        txt = ("Can't instanciate the field '{field_name}', "
+               "'{field_type}' type unknown")
         raise error.InternalError(
             txt.format(field_name=name, field_type=data['type']))
     return field
